/***********************************************************************
 * $Author: markv $
 * $Revision: 1.1 $
 * $Date: 88/09/11 11:00:42 $
 * $Log:	poly.c,v $
 * Revision 1.1  88/09/11  11:00:42  markv
 * Initial revision
 * 
 ***********************************************************************/

#include <stdio.h>
#include <math.h>
#include "defs.h"
#include "extern.h"

typedef struct t_polydata {
	int 	poly_npoints ;
	Vec	* poly_point ;
	Vec	poly_normal ;
	Flt 	poly_d ;
	Flt	poly_p1, poly_p2 ;
} PolyData ;

int PolyPrint ();
int PolyIntersect ();
int PolyNormal ();

ObjectProcs PolyProcs = {
	PolyPrint,
	PolyIntersect,
	PolyNormal,
} ;

int 
PolyPrint(obj)
 Object * obj ;
{
	int i ;
	PolyData * pd ;

	pd = (PolyData *) obj -> o_data ;
	printf("p %d\n", pd -> poly_npoints) ;
	for (i = 0 ; i < pd -> poly_npoints ; i++) {
		printf("%g %g %g\n", pd -> poly_point[i][0],
					pd -> poly_point[i][1],
					pd -> poly_point[i][2]) ;
	}
}

PolyIntersect(obj, ray, hit)
 Object * obj ;
 Ray * ray ;
 Isect * hit ;
{
	Flt n,d,t,m,b ;
	Point V ;
	int i, j, l ;
	int qi, qj ;
	int ri, rj ;
	int c1, c2 ;
	PolyData * pd ;

	pd = (PolyData *) obj -> o_data ;
	n = VecDot(ray -> P, pd -> poly_normal) + pd -> poly_d ;
	d = VecDot(ray -> D, pd -> poly_normal) ;

	if ((Flt) fabs(d) < rayeps) {
		return(NULL);
	}

	t = - n / d ;
	if (t < rayeps)
		return 0 ;

	RayPoint(ray,t,V);
	c1 = pd -> poly_p1 ;
	c2 = pd -> poly_p2 ;

	l = 0 ;
	for (i = 0 ; i < pd -> poly_npoints ; i++) {

		j = (i + 1) % pd -> poly_npoints ;

		qi = 0 ; qj = 0 ;
		ri = 0 ; rj = 0 ;

		if (pd -> poly_point[i][c2] == pd -> poly_point[j][c2])
			continue ;		/*ignore horizontal lines */

		if (pd -> poly_point[i][c2] < V[c2])
			qi = 1 ;
		if (pd -> poly_point[j][c2] < V[c2])
			qj = 1 ;
		if (qi == qj)
			continue ;

		if (pd -> poly_point[i][c1] < V[c1])
			ri = 1 ;
		if (pd -> poly_point[j][c1] < V[c1])
			rj = 1 ;

		if (ri & rj) {
			l++ ;
			continue ;
		}

		if ((ri|rj) == 0)
			continue ;


		/* 
		 * more difficult acceptance...
		 */
		m = (pd -> poly_point[j][c2] - pd -> poly_point[i][c2]) /
			(pd -> poly_point[j][c1] - pd -> poly_point[i][c1]) ;
		
		b = (pd -> poly_point[j][c2] - V[c2]) - 
			m * (pd -> poly_point[j][c1] - V[c1]);

		if ((-b/m) < rayeps)
			l++ ;
	}

	if ((l % 2) == 0)
		return(0);
	
	hit -> isect_t = t ;
	hit -> isect_surf = obj -> o_surf ;
	hit -> isect_prim = obj ;
	hit -> isect_enter = 0 ;
	return(1);
}

PolyNormal(obj, hit, P, N)
 Object * obj ;
 Isect * hit ;
 Point P, N ;
{

	PolyData * pd ;
	pd = (PolyData *) obj -> o_data ;
	VecCopy(pd -> poly_normal, N);
}

Object *
MakePoly(npoints, points)
 int npoints ;
 Vec * points ;
{
	Object * obj ;
	PolyData * pd ;
	Vec P1, P2 ;
	Flt d, dmax, dmin ;
	int i, j ;

	obj = (Object *) malloc (sizeof(Object)) ;
	obj -> o_type = T_POLY ;
	obj -> o_procs = & PolyProcs ;
	obj -> o_surf = CurrentSurface ;

	pd = (PolyData *) malloc (sizeof(PolyData)) ;
	pd -> poly_npoints = npoints ;
	pd -> poly_point = points ;

	/*
	 * calculate the normal by giving various cross products...
	 */
	
	VecSub(pd -> poly_point[0], pd -> poly_point[1], P1) ;
	VecSub(pd -> poly_point[2], pd -> poly_point[1], P2) ;

	VecCross(P1, P2, pd -> poly_normal) ;
	VecNormalize(pd -> poly_normal) ;

	if (fabs(pd -> poly_normal[0]) > fabs(pd -> poly_normal[1])
		&& fabs(pd -> poly_normal[0]) > fabs(pd -> poly_normal[2])) {
		pd -> poly_p1 = 1 ;
		pd -> poly_p2 = 2 ;
	} else if (fabs(pd -> poly_normal[1]) > fabs(pd -> poly_normal[0]) 
		&& fabs(pd -> poly_normal[1]) > fabs(pd -> poly_normal[2])) {
		pd -> poly_p1 = 0 ;
		pd -> poly_p2 = 2 ;
	} else {
		pd -> poly_p1 = 0 ;
		pd -> poly_p2 = 1 ;
	}

	pd -> poly_d = - VecDot(pd -> poly_normal, pd -> poly_point[0]) ;

	obj -> o_data = (void *) pd ;

	/*
	 * now, calculate the values of 
	 * the dmin and dmax 'es for the globally defined slabs...
	 */
	
	for (i = 0 ; i < NSLABS ; i ++) {
		dmin = HUGE ;
		dmax = - HUGE ;

		for (j = 0 ; j < pd -> poly_npoints ; j ++) {
			d = VecDot(Slab[i], pd -> poly_point[j]) ;
			if (d < dmin) dmin = d ;
			if (d > dmax) dmax = d ;
		}
		obj -> o_dmin[i] = dmin ;
		obj -> o_dmax[i] = dmax ;
	}
	return(obj) ;
}
