/***********************************************************************
 * $Author: markv $
 * $Revision: 1.2 $
 * $Date: 88/09/12 10:00:54 $
 * $Log:	screen.c,v $
 * Revision 1.2  88/09/12  10:00:54  markv
 * Fixed lingering (possible) bug with curbuf memory allocation.  
 * Size returned by malloc is now correct in both spots.
 * Thanks tim@ben.
 * 
 * Revision 1.1  88/09/11  11:00:43  markv
 * Initial revision
 * 
 ***********************************************************************/
#include <stdio.h>
#include <math.h>
#include "defs.h"
#include "pic.h"
#include "extern.h"

Screen(view, picfile, xres, yres)
 Viewpoint *view ;
 char *picfile ;
 int xres, yres ;
{
	Pixel 	*buf, *oldbuf, *curbuf, *tmp ;
	Pic  	*pic, *PicOpen();
	Point 	viewvec, leftvec ;
	int red, green, blue ;
	Point	dir ;
	Ray 	ray ;
	Color	color;
	Flt	frustrumwidth ;
	int 	x, y ;

	/*
	 * open the picture file...
	 */

	pic = PicOpen(picfile, xres, yres) ;

	/*
	 * allocate enough space for an entire row of pixels...
	 * plus one more for the average...
	 */

	oldbuf = NULL ;
	curbuf = (Pixel *) malloc ((xres+1) * sizeof (Pixel)) ;
	buf = (Pixel *) malloc ((xres+1) * sizeof (Pixel)) ;

	/*
	 * determine the viewing frustrum
	 */

	VecNormalize(view -> view_up);
	VecSub(view -> view_at, view-> view_from, viewvec);
	VecNormalize(viewvec);
	VecCross(view -> view_up, viewvec, leftvec);
	VecNormalize(leftvec);

	VecCopy(view -> view_from, ray.P);

	frustrumwidth = (view -> view_dist) * ((Flt) tan(view -> view_angle)) ;

	for (y=0; y < yres + 1; y++) {
		for (x = 0; x < xres + 1; x++) {
			VecComb(-frustrumwidth*(2.0*(Flt)y/(Flt)(yres-1)-1.0),
				view -> view_up,
				frustrumwidth*(2.0*(Flt)x/(Flt)(xres-1)-1.0),
				leftvec, dir);
			VecAdd(dir, viewvec, ray.D);
			VecNormalize(ray.D);
			Trace(0, 1.0, &ray, color);

			if (color[0] > 1.0) color[0] = 1.0 ;
			if (color[1] > 1.0) color[1] = 1.0 ;
			if (color[2] > 1.0) color[2] = 1.0 ;

			curbuf[x].r = (unsigned char) (255.0 * color[0]) ;
			curbuf[x].g = (unsigned char) (255.0 * color[1]) ;
			curbuf[x].b = (unsigned char) (255.0 * color[2]) ;
		}
		if (oldbuf) {
			/* average the pixels, and write 'em out
			 */
			for (x = 0 ; x < xres ; x++) {
				red = ((int)curbuf[x].r + (int)curbuf[x+1].r
				     + (int)oldbuf[x].r+ (int)oldbuf[x+1].r) ;
				green = ((int)curbuf[x].g + (int)curbuf[x+1].g
				     + (int)oldbuf[x].g+ (int)oldbuf[x+1].g) ;
				blue = ((int)curbuf[x].b + (int)curbuf[x+1].b
				     + (int)oldbuf[x].b+ (int)oldbuf[x+1].b) ;
				buf[x].r = (unsigned char) (red / 4) ;
				buf[x].g = (unsigned char) (green / 4) ;
				buf[x].b = (unsigned char) (blue / 4) ;
			}
			PicWriteLine(pic, buf);
			tmp = oldbuf ;
			oldbuf = curbuf ;
			curbuf = tmp ;
		} else {
			oldbuf = curbuf ;
			curbuf = (Pixel *) malloc ((xres + 1) * sizeof (Pixel)) ;
		}

		if (tickflag)
			fprintf(stderr, ".") ;
	}
	if (tickflag) fprintf(stderr, "\n") ;
	PicClose(pic);
}
